@@ -1,17 +1,22 @@
 import re
-import warnings
 
 from os import environ
-from collections import MutableMapping
+from collections.abc import MutableMapping
+
+from configDmanager.errors import ReinterpretationError
+from configDmanager._format import FileReader
 
 
 class Config(MutableMapping):
-    def __init__(self, config_dict: dict, parent: 'Config' = None, name: str = None):
+    def __init__(self, config_dict: dict, parent: 'Config' = None, name: str = None, path=None):
         self.__config_dict = dict()
         self.__parent = parent
         self.__load_config_dict(config_dict)
         if name:
             self.set_value('__name', name)
+        else:
+            self.__name = None
+        self.__file_reader = FileReader(path)
 
     def get_name(self):
         return self.__name
@@ -37,7 +42,10 @@ class Config(MutableMapping):
                 try:
                     value = self[match.group(1)]
                 except KeyError:
-                    value = match.group(0)
+                    if match.group(1).startswith('os_environ') or match.group(1).startswith('read_file'):
+                        value = match.group(0)
+                    else:
+                        raise KeyError(match.group(1))
                 text = re.sub(match.group(0), value, text)
         return text
 
@@ -52,7 +60,7 @@ class Config(MutableMapping):
         try:
             return self[item]
         except KeyError:
-            raise AttributeError
+            self.__getattribute__(item)
 
     def __setattr__(self, key, value):
         if not key.startswith(self.__private_prefix):
@@ -68,20 +76,38 @@ class Config(MutableMapping):
 
         try:
             return self.__get_single_item(k)
-        except KeyError:
+        except KeyError as e:
             if self.__parent:
-                return self.__parent.__get_single_item(k)
-            raise KeyError
+                try:
+                    return self.__parent.__get_single_item(k)
+                except KeyError:
+                    pass
+            raise KeyError(f'Could not find param {e} in {self.__name if self.__name else "config"}')
 
     def __get_single_item(self, sub_attributes):
         value = self.__get_raw_single_item(sub_attributes)
         if isinstance(value, str):
-            try:
-                value = self.format_string(value).format(os_environ=environ)
-            except RecursionError:
-                warnings.warn('Two config entries are calling each other: the raw value was used')
-            except KeyError:
-                warnings.warn('Key not found: the raw value was used')
+            value = self.__reinterpret_single_item(sub_attributes, value)
+        return value
+
+    def __reinterpret_single_item(self, sub_attributes, value):
+        try:
+            value = self.format_string(value)
+        except RecursionError:
+            raise ReinterpretationError(sub_attributes, value, 'Due to cycle - RecursionError', RecursionError)
+        except KeyError as e:
+            raise ReinterpretationError(sub_attributes, value,
+                                        f"Could not find param {e} in FstringConfig", KeyError)
+        except FileNotFoundError as e:
+            raise ReinterpretationError(sub_attributes, value, e, FileNotFoundError)
+
+        try:
+            value = value.format(os_environ=environ, read_file=self.__file_reader)
+        except KeyError as e:
+            raise ReinterpretationError(sub_attributes, value,
+                                        f'Could not find {e} in Environment variables', KeyError)
+        except FileNotFoundError as e:
+            raise ReinterpretationError(sub_attributes, value, e, FileNotFoundError)
         return value
 
     def __get_raw_single_item(self, sub_attributes):
